<?php
    require_once KCONFIG . '/classes/classKGatewayUtil.php';
    require_once KCONFIG . '/classes/KGatewayYaml.php';

    $AjaxFunctions['showCodecProfiles'] = 'showCodecProfiles';
    $AjaxFunctions['addCodecProfile'] = 'addCodecProfile';
    $AjaxFunctions['editCodecProfile'] = 'editCodecProfile';
    $AjaxFunctions['saveCodecProfile'] = 'saveCodecProfile';
    $AjaxFunctions['deleteCodecProfile'] = 'deleteCodecProfile';

    function showCodecProfiles( )
    {
        require_once KROOT . '/classes/KLicense.php';

        $app = KSystem::getInstance( );
        $kyaml = new KGatewayYaml( );
        $yaml = $kyaml->getYaml( );

        $buffer = "<fieldset><legend>{$app->translate( 'gateway-codec-profile' )}</legend><div id='gateway-codec-profile-container'><input type='button' id='gateway-codec-profile-button-new' value='{$app->translate( 'gateway-codec-profile-new' )}' /><br /><br />";

        if( array_key_exists( 'CodecProfiles', $yaml ) && count( $yaml['CodecProfiles'] ) > 0 )
        {
            $buffer .= "<table id='gateway-codec-profile-table' class='gateway-table'><thead><tr><th>{$app->translate( 'gateway-codec-profile-name' )}</th><th>{$app->translate( 'gateway-codec-profile-list' )}</th><th class='gateway-table-options'>{$app->translate( 'options' )}</th></tr></thead>";
            foreach( $kyaml->getAllCodecProfiles() as $profile )
            {
                $codecs = array( );
                foreach( $profile['CodecList'] as $codec )
                {
                    if( $codec == 'G729A' && $kyaml->isModel( KGatewayYaml::KMG3200 ) )
                    {
                        $codecs[] = '<span class=\'unavailable\'>G729A</span>';
                    }
                    else
                    {
                        $codecs[] = $codec;
                    }
                }
                $codecList = implode( ',', $codecs );
                $buffer .= "<tr><td>{$profile['Name']}</td><td>{$codecList}</td>";
                $buffer .= "<td class='gateway-table-options'><img class='gateway-codec-img gateway-codec-profile-edit' src='/imagens/editark.gif' title='{$app->translate( 'edit' )}' /><img class='gateway-codec-img gateway-codec-profile-delete' src='/imagens/lixeira.gif' title='{$app->translate( 'delete' )}' /></td></tr>";
            }
            $buffer .= '</table>';
        }
        else
        {
            $buffer .= "<p>{$app->translate( 'gateway-codec-profile-empty-list' )}</p>";
        }

        $buffer .= '</div></fieldset>';
        return $buffer;
    }

    function addCodecProfile( )
    {
        return _codecProfileForm( );
    }

    function editCodecProfile( )
    {
        $name = array_key_exists( 'name', $_POST ) ? $_POST['name'] : null;
        $app = KSystem::getInstance( );

        if( !is_null( $name ) )
        {
            return _codecProfileForm( $name );
        }

        return $app->translate( 'gateway-codec-profile-error-not-found' );
    }

    function saveCodecProfile( )
    {
        require_once KROOT . '/json_util.php';
        $app = KSystem::getInstance( );

        $reply = array( 'success' => false, 'message' => $app->translate( 'gateway-codec-profile-save-error' ) );

        try
        {
            //TODO: change name to Name and codecList to CodecList
            if( array_key_exists( 'name', $_POST ) && array_key_exists( 'oldName', $_POST ) && array_key_exists( 'codecList', $_POST ) )
            {
                $kyaml = new KGatewayYaml( );
                $oldName = isset( $_POST['oldName'] ) ? $_POST['oldName'] : '';
                $kyaml->setCodecProfile( array( 'Name' => $_POST['name'], 'CodecList' => $_POST['codecList'] ), $oldName );
                $kyaml->save( );
                $reply = array( 'success' => true, 'message' => $app->translate( 'gateway-codec-profile-saved' ) );
            }
        }
        catch( Exception $e )
        {
            $reply['message'] = $app->translate( $e->getMessage( ) );
        }

        return JSONencode( $reply );
    }

    function deleteCodecProfile( )
    {
        require_once KROOT . '/json_util.php';
        $app = KSystem::getInstance( );

        $reply = array( 'success' => false, 'message' => $app->translate( 'gateway-codec-profile-delete-error' ) );

        try
        {
            $kyaml = new KGatewayYaml( );
            $kyaml->deleteCodecProfile( $_POST['name'] );
            $kyaml->save( );
            $reply = array( 'success' => true, 'message' => $app->translate( 'gateway-codec-profile-deleted' ) );
        }
        catch( Exception $e )
        {
            $reply['message'] = $app->translate( $e->getMessage( ) );
        }

        return JSONencode( $reply );
    }

    function _codecProfileForm( $name = null )
    {
        require_once KROOT . '/classes/KLicense.php';

        $app = KSystem::getInstance( );
        $kyaml = new KGatewayYaml( );

        $codecProfile = $name ? $kyaml->getCodecProfile( $name ) : array( 'Name' => '', 'CodecList' => array() );

        //TODO: exception if not found?
        if( $name && is_null( $codecProfile ) )
        {
            return $app->translate( 'gateway-codec-profile-error-not-found' );
        }
        $codecList = &$codecProfile['CodecList'];

        //TODO: Codecs list *should* be available from some global source
        // (KGatewayUtil,KGatewayYaml,KSystemYaml (sip configuration for k3l)?)
        if( $kyaml->isModel( KGatewayYaml::KMG3200 ) )
        {
            $codecs = array( 'G711A', 'G711U', 'GSM', 'DVI4', 'iLBC' );
            foreach( $codecList as $idx=>$codec )
            {
                if( $codec === 'G729A' )
                {
                    trigger_error( "Codec {$codec} removed from CODEC profile {$name} list.", E_USER_NOTICE );
                    unset( $codecList[$idx] );
                }
            }
        }
        else
        {
            $codecs = array( 'G711A', 'G711U', 'G729A', 'GSM', 'DVI4', 'iLBC' );
        }

        $buffer = '<form name=\'gateway-codec-profile-form\' id=\'gateway-codec-profile-form\'>';
        $buffer .= '<table id=\'gateway-codec-profile-form-table\'>';
        $buffer .= "<tr><td>{$app->translate( 'gateway-codec-profile-name' )}:</td><td><input type='hidden' id='gateway-codec-profile-old-name' value='{$name}' /><input type='text' id='gateway-codec-profile-name' value='{$name}' /></td></tr>";
        $buffer .= "<tr><td>{$app->translate( 'gateway-codec-profile-list' )}: <img src='/imagens/tooltip.png' class='gateway-tooltip' alt='tooltip' khelp='{$app->translate( 'gateway-codec-profile-help-list' )}' /><br /><a id='gateway-codec-profile-select-all' href='#'>{$app->translate( 'all' )}</a> - <a id='gateway-codec-profile-select-none' href='#'>{$app->translate( 'none' )}</a></td><td><ul class='ui-sortable' id='gateway-codec-profile-list-container'>";
        // reorder $codecs using the $codecList as first elements
        if( count( $codecList ) > 0 )
        {
            $codecsDisabled = array_diff( $codecs, $codecList );
            $codecs = array_merge( $codecList, $codecsDisabled );
            unset( $codecsDisabled );
        }

        foreach( $codecs as $codec )
        {
            if( $codec === 'G729A' )
            {
                $extra = "&nbsp;<img class='gateway-tooltip' khelp='{$app->translate( 'gateway-codec-profile-help-license-required' )}' alt='tooltip' src='/imagens/tooltip.png' />";
            }
            else
            {
                $extra = '';
            }
            $checked = in_array( $codec, $codecList ) ? ' checked' : '';
            $buffer .= "<li class='ui-state-default'><input type='checkbox' class='gateway-codec-profile-list' value='{$codec}'{$checked}>{$codec}{$extra}</li>";
        }
        $buffer .= "</ul></td></tr>";
        $buffer .= "</table><p><input type='button' id='gateway-codec-profile-save' value='{$app->translate( 'save' )}' /><input type='button' id='gateway-codec-profile-back' value='{$app->translate( 'back' )}' /></p></form>";

        return $buffer;
    }
