<?php
    require_once '../constants.php';
    require_once KROOT . '/classes/KSystem.php';
    $app = KSystem::getInstance( );
    $app->redirectIfNotLogged( );

    require_once KROOT . '/smarty-libs/Smarty.class.php';
    require_once KROOT . '/topo.php';
    require_once KROOT . '/rodape.php';
    require_once KROOT . '/util.php';
    require_once KCONFIG . '/languages/dictionary.php';
    require_once KCONFIG . '/classes/classKSistema.php';

    if( !isset( $_SESSION['sis'] ) )
    {
        $sistema = new KSistema( $app->getLanguage( ) );
        $_SESSION['sis'] = serialize( $sistema );
    }
    else
    {
        $sistema = unserialize( $_SESSION['sis'] );
        $sistema->setLanguage( $app->getLanguage( ) );
        $_SESSION['sis'] = serialize( $sistema );
    }

    $smarty = new Smarty( );
    $smarty->assign( 'FOOTER', Rodape::printRodape( ) );

    require_once KCONFIG . '/ajax/includeAjaxFiles.php';

    $ajaxTo = isset( $_REQUEST['ajaxTo'] ) ? $_REQUEST['ajaxTo'] : NULL;
    if( $ajaxTo )
    {
        // test if function exists, then call
        if( function_exists( $AjaxFunctions[$ajaxTo] ) )
        {
            echo call_user_func_array( $AjaxFunctions[$ajaxTo], array( $_REQUEST ) );
        }
        else
        {
            echo "Undefined function {$ajaxTo}";
        }
        exit ;
    }
    $smarty->assign( 'MODULO_PAI', '..' );
    $smarty->assign( 'MODULO', '.' );

    $location = isset( $_REQUEST['location'] ) ? $_REQUEST['location'] : NULL;

    if( $location )
    {
        switch( $location )
        {
            case 'admin':
                $smarty->assign( 'LEGEND', $app->translate( 'administration' ) );
                $topo = Topo::printTopo( 'administration', $_SESSION['loginAccess']['configuration'], $_SESSION['loginAccess']['monitoring'], $_SESSION['loginAccess']['diagnostic'], $_SESSION['loginAccess']['administration'] );
                $smarty->assign( 'TOP', $topo );
                $smarty->assign( 'MODULE', 'Administration' );

                if( $app->hasPermission( 'administration' ) )
                {
                    $smarty->assign( 'FORM', printAdminInterface( $_REQUEST ) );
                }
                else
                {
                    $smarty->assign( 'FORM', printSelfAdministration( $_REQUEST ) );
                }

                applyJSTimeStamp( $smarty );

                $smarty->display( 'themes/default/default_no_menu.tpl' );
                break;
            case 'downloadLicense':
                $name = array_key_exists( 'name', $_GET ) ? $_GET['name'] : null;
                if( is_null( $name ) )
                {
                    header( 'HTTP/1.0 500 Internal Server Error: No name' );
                }

                require_once KROOT . '/classes/KLicense.php';

                try
                {
                    $manager = new KLicenseManager( );
                    $data = $manager->getLicenseRaw( $name );

                    $size = strlen( $data );
                    header( 'Content-Type: application/octect-stream' );
                    header( "Content-Disposition: attachment; filename=\"{$name}\"" );
                    header( 'Content-Transfer-Encoding: binary' );
                    header( "Content-Length: {$size}" );
                    header( 'Expires: 0' );

                    if( array_key_exists( 'HTTP_USER_AGENT', $_SERVER ) && (strpos( $_SERVER['HTTP_USER_AGENT'], 'MSIE' ) !== false) )
                    {
                        header( 'Cache-Control: must-revalidate, post-check=0, pre-check=0' );
                        header( 'Pragma: public' );
                    }
                    else
                    {
                        header( 'Pragma: no-cache' );
                    }

                    echo $data;
                }
                catch( KLicenseManagerException $e )
                {
                    header( "HTTP/1.0 500 Internal Server Error: {$e->getMessage()}" );
                }
                catch( KLicense $e )
                {
                    header( "HTTP/1.0 404 Not Found: {$e->getMessage()}" );
                }
                break;
            case 'downloadShellKey':
                $data = file_get_contents( '/etc/khomp/key.konchasshd' );
                if( $data === false )
                {
                    header( "HTTP/1.0 404 Not Found" );
                    echo $app->translate( 'remote-shell-key-not-found' );
                    break;
                }
                $name = php_uname( 'n' ) . '.key';
                $size = strlen( $data );
                header( 'Content-Type: application/octect-stream' );
                header( "Content-Disposition: attachment; filename=\"{$name}\"" );
                header( 'Content-Transfer-Encoding: binary' );
                header( "Content-Length: {$size}" );
                header( 'Expires: 0' );

                if( array_key_exists( 'HTTP_USER_AGENT', $_SERVER ) && (strpos( $_SERVER['HTTP_USER_AGENT'], 'MSIE' ) !== false) )
                {
                    header( 'Cache-Control: must-revalidate, post-check=0, pre-check=0' );
                    header( 'Pragma: public' );
                }
                else
                {
                    header( 'Pragma: no-cache' );
                }

                echo $data;
                break;
        }
    }
    else
    {
        $smarty->assign( 'LEGEND', $app->translate( 'sisConfig' ) );
        $topo = Topo::printTopo( 'config', $_SESSION['loginAccess']['configuration'], $_SESSION['loginAccess']['monitoring'], $_SESSION['loginAccess']['diagnostic'], $_SESSION['loginAccess']['administration'], '..' );
        $smarty->assign( 'TOP', $topo );
        $smarty->assign( 'CONFIG_SAVED', $app->translate( 'configurationSaved' ) );

        if( $app->hasPermission( 'configuration' ) )
        {
            $menu = call_user_func_array( $AjaxFunctions['showTreeMenu'], array( $_REQUEST ) );
            $smarty->assign( 'MENU', $menu );

            if( IS_GATEWAY )
            {
                $smarty->assign( 'CHANGEVIEW', 'showRoutesGW' );
                $smarty->assign( 'SECTION', 'menu-routing-gw,menu-routes-gw' );
                $smarty->assign( 'FORM', '' );
            }
            else
            {
                $smarty->assign( 'CHANGEVIEW', 'showDevices' );
                $smarty->assign( 'SECTION', 'menu-device' );
                $smarty->assign( 'FORM', showDevices( $_REQUEST ) );
            }
        }
        else
        {
            $smarty->assign( 'MENU', '' );
            $smarty->assign( 'CHANGEVIEW', '' );
            $smarty->assign( 'SECTION', '' );
            $smarty->assign( 'FORM', $app->translate( 'pDenied' ) );
        }

        //js conflict with elastix framework
        $jq = filemtime( '../js/jquery/jquery.min.js' );
        $js_conflict = '<script type="text/javascript" src="/js/jquery/jquery.min.js?' . $jq . '"></script>';
        $jmin = filemtime( '../js/jquery/jquery-ui-1.8.14.custom.min.js' );
        $js_conflict .= '<script type="text/javascript" src="/js/jquery/jquery-ui-1.8.14.custom.min.js?' . $jmin . '"></script>';
        $smarty->assign( 'JS_CONFLICT', $js_conflict );

        applyJSTimeStamp( $smarty );

        $smarty->display( 'themes/default/default.tpl' );
    }

    function applyJSTimeStamp( $smarty )
    {
        //ataching timestamp to prevent cache
        $smarty->assign( 'PRINCIPAL_CSS_TS', filemtime( './css/principal.css' ) );
        $smarty->assign( 'GERAL_JS_TS', filemtime( '../js/geral.js' ) );
        $smarty->assign( 'GERAL_CSS_TS', filemtime( '../js/geral.css' ) );
        $smarty->assign( 'JQUERY_JS_TS', filemtime( '../js/jquery/jquery.min.js' ) );
        $smarty->assign( 'KWEBCONFIG_JS_TS', filemtime( './js/kwebconfig.js' ) );
        $smarty->assign( 'KGATEWAY_JS_TS', filemtime( './js/kgateway.js' ) );

        $smarty->assign( 'UI_CSS_TS', filemtime( './css/jquery.ui.all.css' ) );
        $smarty->assign( 'TREEVIEW_CSS_TS', filemtime( './css/treeMenu/jquery.treeview.css' ) );

        $smarty->assign( 'TREEVIEW_JS_TS', filemtime( '../js/jquery/jquery.treeview.js' ) );
        $smarty->assign( 'TREE_JS_TS', filemtime( './css/treeMenu/tree.js' ) );
        $smarty->assign( 'VALIDATE_JS_TS', filemtime( '../js/jquery/jquery-1.6.validate.js' ) );
        $smarty->assign( 'DRAGGABLE_JS_TS', filemtime( '../js/jquery/jquery.ui.draggable.js' ) );
        $smarty->assign( 'SORTABLE_JS_TS', filemtime( '../js/jquery/jquery.ui.sortable.js' ) );
        $smarty->assign( 'WIDGET_JS_TS', filemtime( '../js/jquery/jquery.ui.widget.js' ) );
        $smarty->assign( 'COLLAPSER_JS_TS', filemtime( '../js/jquery/jquery.collapser.min.js' ) );
        $smarty->assign( 'TINPUTS_JS_TS', filemtime( '../js/jquery/jquery.textinputs.js' ) );
        $smarty->assign( 'TABLESORT_JS_TS', filemtime( '../js/jquery/jquery.tablesorter.min.js' ) );
        $smarty->assign( 'MASKINPUT_JS_TS', filemtime( '../js/jquery/jquery.maskedinput-1.3.min.js' ) );
        $smarty->assign( 'IP_ADDR_JS_TS', filemtime( '../js/jquery/jquery.input-ip-address-control-1.0.min.js' ) );
        $smarty->assign( 'JFORM_JS_TS', filemtime( '../js/jquery/jquery.form.js' ) );
        $smarty->assign( 'DROPDOWN_JS_TS', filemtime( '../js/jquery/ui.dropdownchecklist-1.4-min.js' ) );
    }
