<?php
    require_once KCONFIG . '/classes/classKGatewayUtil.php';
    require_once KCONFIG . '/classes/classKGatewayNetwork.php';
    require_once KCONFIG . '/classes/KGatewayYaml.php';

    $AjaxFunctions['showNetworkGW'] = 'showNetworkGW';
    $AjaxFunctions['saveNetworkGW'] = 'saveNetworkGW';
    $AjaxFunctions['showVoIPOptionsGW'] = 'showVoIPOptionsGW';

    define( 'GATEWAY_NETWORK_CONFIG', '/etc/network/interfaces' );
    define( 'GATEWAY_NETWORK_DEV', 'eth1' );

    function showNetworkGW( )
    {

        $app = KSystem::getInstance( );
        $yaml = new KGatewayYaml( );

        $buffer = '';

        $buffer .= "<fieldset><legend>{$app->translate( 'gateway-network' )}</legend>";
        $buffer .= "<div id='gateway-network'>";

        if( $yaml->isModel( KGatewayYaml::KMG3200 ) )
        {
            $ifaces = array( 'eth0', 'eth1' );
        }
        else
        {
            $ifaces = array( GATEWAY_NETWORK_DEV );
        }

        try
        {
            $parser = new KGatewayNetwork( GATEWAY_NETWORK_CONFIG );
            $parser->load( );

            foreach( $ifaces as $idx=>$iface )
            {
                try
                {
                    $properties = $parser->getProperties( $iface );
                    $config = $parser->getConfiguration( $iface );
                    $macaddress = $parser->getMacAddress( $iface );
                }
                catch( Exception $e )
                {
                    if( $yaml->isModel( KGatewayYaml::KMG3200 ) )
                    {
                        trigger_error( "{$iface}: {$e->getMessage()}", E_USER_NOTICE );
                        continue;
                    }
                    throw $e;
                }

                $address = array_key_exists( 'address', $config ) ? $config['address'] : '';
                $netmask = array_key_exists( 'netmask', $config ) ? $config['netmask'] : '';
                $gateway = array_key_exists( 'gateway', $config ) ? $config['gateway'] : '';

                $buffer .= "<div id='gateway-network-{$iface}' class='gateway-network-interface'>";
                if( $yaml->isModel( KGatewayYaml::KMG3200 ) )
                {
                    $buffer .= "<fieldset><legend>{$app->translate( 'gateway-network-interface' )} {$iface} [{$app->translate( 'gateway-network-interface-macaddress' )} {$macaddress}]</legend>";
                }
                else
                {
                    $buffer .= "<fieldset><legend>{$app->translate( 'gateway-network-interface' )} [{$app->translate( 'gateway-network-interface-macaddress' )} {$macaddress}]</legend>";
                }

                $buffer .= "<div class='gateway-network-method'><label for='gateway-network-method-{$iface}'>{$app->translate( 'gateway-network-method' )}</label> <select id='gateway-network-method-{$iface}' class='gateway-network-interface-method'>";
                foreach( KGatewayNetwork::$methods[$properties['family']] as $method )
                {
                    // cannot disable first interface 
                    if( $idx == 0 && $method == 'manual' )
                    {
                        continue;
                    }
                    $selected = $properties['method'] == $method ? ' selected=\'true\'' : '';
                    $name = $app->translate( "gateway-network-method-{$method}" );
                    $buffer .= "<option value='{$method}'{$selected}>{$name}</option>";
                }
                $buffer .= '</select></div><br />';
                $buffer .= "<div id='gateway-network-config-dhcp-{$iface}' style='display: none;'>";
                $buffer .= '<table><tbody>';
                $buffer .= '</tbody></table>';
                $buffer .= "</div>";
                $buffer .= "<div id='gateway-network-config-static-{$iface}' style='display: none;'>";
                $buffer .= "<table>";
                $buffer .= "<tr><td><label for='gateway-network-static-address-{$iface}'>{$app->translate( 'gateway-network-address' )}</label></td><td><input id='gateway-network-static-address-{$iface}' class='ipv4' type='text' value='{$address}' /></td></tr>";
                $buffer .= "<tr><td><label for='gateway-network-static-netmask-{$iface}'>{$app->translate( 'gateway-network-netmask' )}</label></td><td><input id='gateway-network-static-netmask-{$iface}' class='ipv4' type='text' value='{$netmask}' /></td></tr>";
                $buffer .= "<tr><td><label for='gateway-network-static-gateway-{$iface}'>{$app->translate( 'gateway-network-gateway' )}</label></td><td><input id='gateway-network-static-gateway-{$iface}' class='ipv4' type='text' value='{$gateway}' /></td></tr>";
                $buffer .= "</table>";
                $buffer .= "</div>";
                unset( $address );
                unset( $netmask );
                unset( $gateway );

                $buffer .= "</fieldset><br />";
            }

            $buffer .= '<div id=\'gateway-network-dns\'><fieldset><legend>DNS</legend>';
            if( $yaml->isModel( KGatewayYaml::KMG3200 ) )
            {
                $buffer .= '<input type=\'hidden\' id=\'gateway-network-dns-method\' value=\'static\' />';
            }
            else
            {
                $buffer .= "<div class='gateway-network-method'><label for='gateway-network-dns-method'>{$app->translate( 'gateway-network-dns-method' )}</label> <select id='gateway-network-dns-method'>";
                foreach( KGatewayNetwork::$dnsmethods as $method )
                {
                    $selected = array_key_exists( 'dns-nameservers', $config ) && $method == 'static' ? ' selected=\'true\'' : '';
                    $name = $app->translate( "gateway-network-method-{$method}" );
                    $buffer .= "<option value='{$method}'{$selected}>{$name}</option>";
                }
                $buffer .= '</select></div><br />';
            }

            $dnssearch = array_key_exists( 'dns-search', $config ) ? $config['dns-search'] : '';
            $buffer .= "<div id='gateway-network-dns-config'><table><tbody>";
            $buffer .= "<tr><td><label for='gateway-network-dns-search'>{$app->translate( 'gateway-network-dns-search' )}</label></td><td><input type='text' id='gateway-network-dns-search' value='{$dnssearch}' /></td></tr>";
            $buffer .= "<tr><td>{$app->translate( 'gateway-network-servers' )}</td><td><div id='gateway-network-dns-server-list'>";
            $servers = array_key_exists( 'dns-nameservers', $config ) ? explode( ' ', $config['dns-nameservers'] ) : array( );
            foreach( $servers as $server )
            {
                $buffer .= "<input type='text' class='gateway-network-dns-server ipv4' value='{$server}' /><br />";
            }
            $buffer .= "</div><a href='#' class='gateway-network-dns-add'>{$app->translate( 'gateway-network-add-server' )}</a></td></tr>";
            $buffer .= '</tbody></table></fieldset></div><br />';
            unset( $dnssearch );

            $buffer .= "<input type='button' id='gateway-network-save' value='{$app->translate( 'save' )}' />";
        }
        catch( KGatewayNetworkException $e )
        {
            $buffer .= $app->translate( $e->getMessage( ) );
        }

        $buffer .= "</div></fieldset>";

        return $buffer;
    }

    function saveNetworkGW( )
    {
        require_once KROOT . '/json_util.php';

        $app = KSystem::getInstance( );
        $reply = array( 'success' => false, 'message' => $app->translate( 'gateway-network-save-error' ) );

        try
        {
            $parser = new KGatewayNetwork( GATEWAY_NETWORK_CONFIG );
            $parser->load( );

            // interfaces configuration
            foreach( $_POST['ifaces'] as $iface => $data )
            {
                $config = array_key_exists( 'config', $data ) ? $data['config'] : array( );
                $parser->setProperties( $iface, $data['properties'], $config );
            }

            $parser->save( );

            $reply = array( 'success' => true, 'message' => $app->translate( 'gateway-network-saved' ) );
        }
        catch( KGatewayNetworkException $e )
        {
            $reply['message'] = $app->translate( $e->getMessage( ) );
        }
        return JSONencode( $reply );
    }

    function showVoIPOptionsGW( $_REQUEST )
    {
        return showVoIPOptions( $_REQUEST, 'GW' );
    }
